/*----------------------------------------------------------------------------
 HMG - Harbour Windows GUI library source code

 Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 http://sites.google.com/site/hmgweb/

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this software; see the file COPYING. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text
 contained in this release of HMG.

 The exception is that, if you link the HMG library with other
 files to produce an executable, this does not by itself cause the resulting
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the
 HMG library code into it.

 Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2008 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include 'hmg.ch'

*-----------------------------------------------------------------------------*
Function GetColor( aInitColor )
*-----------------------------------------------------------------------------*
Local aRetVal [3] , nColor , nInitColor

   if valtype ( aInitColor ) != 'U'
      nInitColor := RGB ( aInitColor [1] , aInitColor [2] , aInitColor [3] )
   EndIf

   nColor := ChooseColor( NIL, nInitColor )

   If nColor == -1
      aRetVal [1] := Nil
      aRetVal [2] := Nil
      aRetVal [3] := Nil
   Else
      aRetVal [1] := GetRed (nColor)
      aRetVal [2] := GetGreen (nColor)
      aRetVal [3] := GetBlue (nColor)
   EndIf

Return aRetVal

*-----------------------------------------------------------------------------*
Function GetFolder (Title)
*-----------------------------------------------------------------------------*
Local folder
   folder := C_GETFOLDER(TITLE)
Return folder

*-----------------------------------------------------------------------------*
Function browseforfolder( nfolder,nflag) // Contributed By Ryszard Rylko
*-----------------------------------------------------------------------------*
Local RetVal:=""
   RetVal := C_browseforfolder( nfolder,nflag)
return RetVal

*-----------------------------------------------------------------------------*
Function GetFile( aFilter, title, cIniFolder, multiselect, nochangedir )
*-----------------------------------------------------------------------------*
local c := ''
local cfiles := ''
local fileslist := {}
local n
Local i

   IF aFilter == Nil
      aFilter := {}
   EndIf

   IF title == NIL
      title := ""
   ENDIF

   IF cIniFolder == NIL
      cIniFolder := ""
   ENDIF

   FOR n:=1 TO LEN(aFilter)
       c :=  c + aFilter[n][1] + chr(0) + aFilter[n][2] + chr(0)
   NEXT
   c :=  c + chr(0)

   if valtype(multiselect) == 'U'
      multiselect := .f.
   endif

   if .not. multiselect
      Return ( C_GetFile ( c , title, cIniFolder, multiselect ,nochangedir ) )
   else
      cfiles := C_GetFile ( c , title, cIniFolder, multiselect ,nochangedir )

      if len( cfiles ) > 0
         if valtype( cfiles ) == "A"
            fileslist := aclone( cfiles )
         else
            aadd( fileslist, cfiles )
         endif
      endif

      For i := 1 To Len ( FilesList)
         FilesList [i] := StrTran ( FilesList [i] , "\\" , "\" )
      Next i

      Return ( FilesList )

   endif

Return Nil

*-----------------------------------------------------------------------------*
Function Putfile ( aFilter, title, cIniFolder, nochangedir )
*-----------------------------------------------------------------------------*
local c:='' , n

   IF aFilter == Nil
      aFilter:={}
   EndIf

   IF title == NIL
      title := ""
   ENDIF

   IF cIniFolder == NIL
      cIniFolder := ""
   ENDIF

   FOR n := 1 TO Len ( aFilter )
      c += aFilter [n] [1] + chr(0) + aFilter [n] [2] + chr(0)
   NEXT
   c :=  c + chr(0)

Return C_PutFile ( c, title, cIniFolder, nochangedir )

*------------------------------------------------------------------------------*
Function GetFont( cInitFontName , nInitFontSize , lBold , lItalic , anInitColor , lUnderLine , lStrikeOut , nCharset )
*------------------------------------------------------------------------------*
Local RetArray [8] , Tmp , rgbcolor

   If ValType ( cInitFontName ) == 'U'
      cInitFontName := ""
   EndIf

   If ValType ( nInitFontSize ) == 'U'
      nInitFontSize := 0
   EndIf

   If ValType ( lBold ) == 'U'
      lBold := .F.
   EndIf

   If ValType ( lItalic ) == 'U'
      lItalic := .F.
   EndIf

   If ValType ( anInitColor ) == 'U'
      rgbcolor := 0
   Else
      rgbcolor := RGB( anInitColor[1] , anInitColor[2] , anInitColor[3] )
   EndIf

   If ValType ( lUnderLine ) == 'U'
      lUnderLine := .F.
   EndIf

   If ValType ( lStrikeOut ) == 'U'
      lStrikeOut := .F.
   EndIf

   If ValType ( nCharSet ) == 'U'
      nCharSet := 0
   EndIf

   RetArray := ChooseFont( cInitFontName , nInitFontSize , lBold , lItalic , rgbcolor , lUnderLine , lStrikeOut , nCharSet )

   If ! Empty ( RetArray [1] )
      Tmp := RetArray [5]
      RetArray [5] := { GetRed(Tmp) , GetGreen(Tmp) , GetBlue(Tmp) }
   Else
      RetArray [5] := { Nil , Nil , Nil }
   EndIf

Return RetArray
